#include "housenka.h"

Grid::Grid(int width, int height) {
		eraseNeeded = false;
		gridColor = RGB(0,0,0);

		this->width = width;
		this->height = height;
	}

	/** Call this method to repaint the grid.
	* @param backgroundColor Background color of a grid. Is used to clear grid if it is reduced.
	* @param g Graphics object used to painting.
	* @param left x-coordinate of upper-left corner of a grid (in pixels).
	* @param top y-coordinate of upper-left corner of a grid (in pixels).
	*/    
	void Grid::Paint(COLORREF backgroundColor, HWND hwnd, int left, int top)
	{
		if (eraseNeeded)
		{
			PaintWithColor(hwnd, oldLeft, oldTop, backgroundColor, oldWidth, oldHeight);
			eraseNeeded = false;
		}
		PaintWithColor(hwnd, left, top, gridColor, this->width, this->height);
	}

	void Grid::PaintWithColor(HWND hwnd, int left, int top, COLORREF c, int width, int height)
	{
		HDC hdc;

		hdc = GetDC(hwnd);

		if (c != RGB(0, 0, 0))// not black, erase
		{
			SelectObject(hdc, GetStockObject(WHITE_PEN));
		}


		int right = left + GetWidthPixels() - 1;
		MoveToEx(hdc, left, top, NULL);
		LineTo(hdc, right, top);
		int y = top + 1;
		for (int i = 0; i < height; i++)
		{
			MoveToEx(hdc, left, y, NULL);
			LineTo(hdc, right, y);
			y += cellSize + 1;
		}
		//        y += cellSize + 1;
		MoveToEx(hdc, left, y, NULL);
		LineTo(hdc, right, y);
		y++;
		MoveToEx(hdc, left, y, NULL);
		LineTo(hdc, right, y);

		int bottom = top + GetHeightPixels() - 1;
		MoveToEx(hdc, left, top, NULL);
		LineTo(hdc, left, bottom);
		int x = left + 1;
		for (int i = 0; i < width; i++)
		{
			MoveToEx(hdc, x, top, NULL);
			LineTo(hdc, x, bottom);
			x += cellSize + 1;
		}
		//        x += cellSize + 1;
		MoveToEx(hdc, x, top, NULL);
		LineTo(hdc, x, bottom);
		x++;
		MoveToEx(hdc, x, top, NULL);
		LineTo(hdc, x, bottom);

		oldLeft = left;
		oldTop = top;

		ReleaseDC(hwnd, hdc);
	}

	/** Gets the width of a grid in pixels.
	* @return The whole width of a grid in pixels.
	*/    
	int Grid::GetWidthPixels()
	{
		return width * (cellSize + 1) + 3;
	}

	/** Gets the height of a grid in pixels.
	* @return The whole height of a grid in pixels.
	*/    
	int Grid::GetHeightPixels()
	{
		return height * (cellSize + 1) + 3;
	}

	/** Sets the width of a grid.
	* @param width Width of a grid (number of cells).
	* @throws IllegalArgumentException Throws if width is less than 1 or greater than 50.
	*/    
	void Grid::SetWidth(int width)
	{
		if ((width < 10) || (width > 50))
		{
			MessageBox(NULL, "Value 10 - 50 required", "error", 0);
			return;
		}
		this->oldWidth = this->width;
		this->width = width;
		eraseNeeded = true;
	}

	/** Sets the height of a grid.
	* @param height Height of a grid (number of cells).
	* @throws IllegalArgumentException Throws if height is less than 1 or greater than 50.
	*/    
	void Grid::SetHeight(int height)
	{
		if ((height < 10) || (height > 50))
		{
			MessageBox(NULL, "Value 10 - 50 required", "error", 0);
			return;
		}
		this->oldHeight = this->height;
		this->height = height;
		eraseNeeded = true;
	}

	/** Gets the width of a grid (number of cells).
	* @return Wdith of a grid.
	*/    
	int Grid::GetWidth()
	{
		return width;
	}

	/** Gets the height of a grid (number of cells).
	* @return Height of a grid.
	*/    
	int Grid::GetHeight()
	{
		return height;
	}

	/** Counts the upper-left corner point of a interior of a specific cell.
	* @param cellX Column number. Zero-based.
	* @param cellY Row number. Zero-based.
	* @param positionX X-coordinate (in pixels) of upper-left corner of a grid.
	* @param positionY Y-coordinate (in pixels) of upper-left corner of a grid.
	* @return upper-left corner point of a interior of a specific cell
	*/    
	POINT * Grid::countCellPosition(int cellX, int cellY, int positionX, int positionY, POINT * p)
	{
		p->x = positionX + 2 + cellX * (cellSize + 1);
		p->y = positionY + 2 + cellY * (cellSize + 1);
		return p;
	}
