#include "housenka.h"

HWND mainWindow;
HMENU menu, actionMenu;
HWND startButton, stopButton, pauseButton;
WNDPROC oldButtonProc, oldEditProc;
HWND widthEdit, heightEdit, speedEdit, barriersEdit;

Grid * grid;
Caterpillar * caterpillar;

bool paused = false;
bool gridResized;
int caterpillarSpeed;
int caterpillarSpeedTime;
int numberOfBarriers;
bool canMove = false; 

#define ID_START_BUTTON 1
#define ID_STOP_BUTTON 2
#define ID_PAUSE_BUTTON 3
#define ID_WIDTH_EDIT 4
#define ID_HEIGHT_EDIT 5
#define ID_SPEED_EDIT 6
#define ID_BARRIERS_EDIT 7
#define ID_ABOUT 8


int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
{
     static TCHAR szAppName[] = TEXT ("housenka");
     HWND         hwnd;
     MSG          msg;
     WNDCLASS     wndclass;

     wndclass.style         = CS_HREDRAW | CS_VREDRAW;
     wndclass.lpfnWndProc   = WndProc;
     wndclass.cbClsExtra    = 0;
     wndclass.cbWndExtra    = 0;
     wndclass.hInstance     = hInstance;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
     wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH);
     wndclass.lpszMenuName  = NULL;
     wndclass.lpszClassName = szAppName;

     if (!RegisterClass(&wndclass))
     {
          MessageBox (NULL, TEXT ("Tento program vyaduje Windows NT!"), 
                      szAppName, MB_ICONERROR);
          return 0;
     }
     
     hwnd = CreateWindow(szAppName,                  // nzev tdy okna
                          TEXT ("Housenka"),          // zhlav okna
                          WS_OVERLAPPEDWINDOW,        // styl okna
                          CW_USEDEFAULT,              // vchoz poloha x
                          CW_USEDEFAULT,              // vchoz poloha y
                          CW_USEDEFAULT,              // vchoz velikost x
                          CW_USEDEFAULT,              // vchoz velikost y
                          NULL,                       // handle rodiovskho okna
                          NULL,                       // handle nabdky okna
                          hInstance,                  // handle instance programu
                          NULL);                     // parametry pro vytvoen
     
		 mainWindow = hwnd;
     ShowWindow(hwnd, iCmdShow);
     UpdateWindow(hwnd);
		 srand((unsigned) time(NULL));
     
     while (GetMessage(&msg, NULL, 0, 0))
     {
          TranslateMessage(&msg);
          DispatchMessage(&msg);
     }
     return (int) msg.wParam;
}

    void SetPauseButtonLabel()
    {
        if (paused)
				{
            SetWindowText(pauseButton, TEXT("Resume"));
						ModifyMenu(actionMenu, ID_PAUSE_BUTTON, MF_BYCOMMAND | MF_STRING | GetMenuState(actionMenu, ID_PAUSE_BUTTON, MF_BYCOMMAND), ID_PAUSE_BUTTON, TEXT("&Resume"));
				}
        else
				{
            SetWindowText(pauseButton, TEXT("Pause"));
						ModifyMenu(actionMenu, ID_PAUSE_BUTTON, MF_BYCOMMAND | MF_STRING | GetMenuState(actionMenu, ID_PAUSE_BUTTON, MF_BYCOMMAND), ID_PAUSE_BUTTON, TEXT("&Pause"));
				}
    }

    void Start() {
        SetTimer(mainWindow, 1, caterpillarSpeedTime, NULL);
    }

    void Stop() {
        KillTimer(mainWindow, 1);
    }

		void repaint()
		{
          RECT rect;
					HDC hdc;
					HBRUSH brush;

					GetClientRect(mainWindow, &rect);
					rect.left = 160;
					brush = CreateSolidBrush(RGB(255, 255, 255));
					hdc = GetDC(mainWindow);
					FillRect(hdc, &rect, brush);
					ReleaseDC(mainWindow, hdc);
					DeleteObject(brush);

					grid->Paint(RGB(255, 255, 255), mainWindow, 160, 10);

					if (gridResized)
					{
            if (caterpillar != NULL)
            {
                caterpillar->Delete(mainWindow);
                caterpillar = NULL;
            }
            gridResized = false;
					}
        
					if (caterpillar != NULL)
            caterpillar->paint(mainWindow); 
		}

    void setEnabledControls(bool b)
    {
        EnableWindow(widthEdit, b);
        EnableWindow(heightEdit, b);
        EnableWindow(speedEdit, b);
        EnableWindow(barriersEdit, b);
    }

		void MoveFocus(HWND hwnd)
		{
			int i;

			HWND arr[7] = {widthEdit, heightEdit, speedEdit, barriersEdit, startButton,
				stopButton, pauseButton };

			i = 0;
			do
			{
				if (hwnd == arr[i])
					break;
				i++;
			} while (i < 7);
			if (i == 7)
				return;

			int j = (i + 1) % 7;
			while (j != i)
			{
				if (IsWindowEnabled(arr[j]))
				{
					SetFocus(arr[j]);
					break;
				}
				j = (j + 1) % 7;
			}

		}

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
     HDC         hdc;
     PAINTSTRUCT ps;
     RECT        rect;
     
     switch (message)
     {
     case WM_CREATE:

			 // Create menu
			 HMENU submenu;
			 menu = CreateMenu();
			 submenu = CreateMenu();
			 AppendMenu(submenu, MF_STRING, ID_START_BUTTON, "S&tart");
			 AppendMenu(submenu, MF_STRING, ID_STOP_BUTTON, "&Stop");
			 AppendMenu(submenu, MF_STRING, ID_PAUSE_BUTTON, "&Pause");

			 AppendMenu(menu, MF_POPUP, (UINT_PTR) submenu, "&Action");
			 actionMenu = submenu;

			 submenu = CreateMenu();
			 AppendMenu(submenu, MF_STRING, ID_ABOUT, "A&bout application...");
			 AppendMenu(menu, MF_POPUP, (UINT_PTR) submenu, "A&bout");

			 SetMenu(hwnd, menu);
			 
			 // Create buttons
			 startButton = CreateWindow(TEXT ("button"), TEXT("Start"), WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
					 10, 130, 140, 30, hwnd, (HMENU) ID_START_BUTTON, ((LPCREATESTRUCT) lParam)->hInstance, NULL);
			 oldButtonProc = (WNDPROC) SetWindowLong(startButton, GWL_WNDPROC, (LONG) MyButtonWndProc);
			 stopButton = CreateWindow(TEXT ("button"), TEXT("Stop"), WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
					 10, 165, 140, 30, hwnd, (HMENU) ID_STOP_BUTTON, ((LPCREATESTRUCT) lParam)->hInstance, NULL);
			 SetWindowLong(stopButton, GWL_WNDPROC, (LONG) MyButtonWndProc);
			 pauseButton = CreateWindow(TEXT ("button"), TEXT("Pause"), WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
					 10, 200, 140, 30, hwnd, (HMENU) ID_PAUSE_BUTTON, ((LPCREATESTRUCT) lParam)->hInstance, NULL);
			 SetWindowLong(pauseButton, GWL_WNDPROC, (LONG) MyButtonWndProc);

			 //Create edit fields
			 widthEdit = CreateWindow(TEXT("edit"), TEXT("20"), WS_CHILD | WS_VISIBLE | WS_BORDER | ES_LEFT,
						100, 10, 50, 25, hwnd, (HMENU) ID_WIDTH_EDIT, ((LPCREATESTRUCT) lParam)->hInstance, NULL);
			 oldEditProc = (WNDPROC) SetWindowLong(widthEdit, GWL_WNDPROC, (LONG) MyEditWndProc);
			 heightEdit = CreateWindow(TEXT("edit"), TEXT("20"), WS_CHILD | WS_VISIBLE | WS_BORDER | ES_LEFT,
						100, 40, 50, 25, hwnd, (HMENU) ID_HEIGHT_EDIT, ((LPCREATESTRUCT) lParam)->hInstance, NULL);
			 SetWindowLong(heightEdit, GWL_WNDPROC, (LONG) MyEditWndProc);
			 speedEdit = CreateWindow(TEXT("edit"), TEXT("5"), WS_CHILD | WS_VISIBLE | WS_BORDER | ES_LEFT,
						100, 70, 50, 25, hwnd, (HMENU) ID_SPEED_EDIT, ((LPCREATESTRUCT) lParam)->hInstance, NULL);
			 SetWindowLong(speedEdit, GWL_WNDPROC, (LONG) MyEditWndProc);
			 barriersEdit = CreateWindow(TEXT("edit"), TEXT("20"), WS_CHILD | WS_VISIBLE | WS_BORDER | ES_LEFT,
						100, 100, 50, 25, hwnd, (HMENU) ID_BARRIERS_EDIT, ((LPCREATESTRUCT) lParam)->hInstance, NULL);
			 SetWindowLong(barriersEdit, GWL_WNDPROC, (LONG) MyEditWndProc);

        numberOfBarriers = 20;
				SendMessage(hwnd, WM_COMMAND, MAKELONG(ID_SPEED_EDIT, EN_KILLFOCUS), (LPARAM) speedEdit);       
        grid = new Grid(20, 20);
        gridResized = true;
        caterpillar = NULL;
        
				SetFocus(startButton);
				EnableWindow(stopButton, false);
				EnableMenuItem(actionMenu, ID_STOP_BUTTON, MF_BYCOMMAND | MF_GRAYED);
				EnableWindow(pauseButton, false);
				EnableMenuItem(actionMenu, ID_PAUSE_BUTTON, MF_BYCOMMAND | MF_GRAYED);

          return 0;
 
		 case WM_SIZE:
			 GetClientRect(hwnd, &rect);
			 rect.bottom = 10;
			 return 0;
			 
     case WM_PAINT:

          hdc = BeginPaint(hwnd, &ps);
          
					// Draw texts:
					SetRect(&rect, 10, 10, 95, 35);
					DrawText(hdc, TEXT ("Width:"), -1, &rect, DT_SINGLELINE | DT_VCENTER);
					SetRect(&rect, 10, 40, 95, 65);
					DrawText(hdc, TEXT ("Height:"), -1, &rect, DT_SINGLELINE | DT_VCENTER);
					SetRect(&rect, 10, 70, 95, 95);
					DrawText(hdc, TEXT ("Speed:"), -1, &rect, DT_SINGLELINE | DT_VCENTER);
					SetRect(&rect, 10, 100, 95, 125);
					DrawText(hdc, TEXT ("Barriers:"), -1, &rect, DT_SINGLELINE | DT_VCENTER);
          
					repaint();

          EndPaint (hwnd, &ps);
          return 0;

		 case WM_COMMAND:
			 switch (LOWORD(wParam))
			 {
			 case ID_ABOUT:
				 MessageBox(mainWindow, TEXT("Created by Pavel Novak, MFF UK, 09/2003"), TEXT("ABOUT"), 0);
				 return 0;
			 case ID_START_BUTTON:
				 switch (HIWORD(wParam))
				 {
				 case BN_CLICKED:
					 EnableWindow(stopButton, true);
						EnableMenuItem(actionMenu, ID_STOP_BUTTON, MF_BYCOMMAND | MF_ENABLED);
					 EnableWindow(startButton, false);
					EnableMenuItem(actionMenu, ID_START_BUTTON, MF_BYCOMMAND | MF_GRAYED);
					 EnableWindow(pauseButton, true);
					EnableMenuItem(actionMenu, ID_PAUSE_BUTTON, MF_BYCOMMAND | MF_ENABLED);
					 setEnabledControls(false);
					 SetFocus(pauseButton);

					 if (caterpillar != NULL)
					 {
						 caterpillar->Delete(mainWindow);
						 delete caterpillar;
					 }
					 caterpillar = new Caterpillar(grid->GetWidth(), grid->GetHeight(), 160, 10, RGB(255, 255, 255), grid, numberOfBarriers, mainWindow);
					 caterpillar->paint(mainWindow);
					 canMove = false;
					 Start();
						repaint();

						RECT rect;
					 HDC hdc;
					 GetClientRect(mainWindow, &rect);
					 hdc = GetDC(mainWindow);
					 TextOut(hdc, 160, grid->GetHeightPixels() + 15, TEXT("Press a key to start a game..."), 30);
					 ReleaseDC(mainWindow, hdc);

					 break;
				 }
				 break;

			 case ID_STOP_BUTTON:
				 switch (HIWORD(wParam))
				 {
				 case BN_CLICKED:

					 Stop();

					 EnableWindow(pauseButton, false);
					EnableMenuItem(actionMenu, ID_PAUSE_BUTTON, MF_BYCOMMAND | MF_GRAYED);
					 EnableWindow(stopButton, false);
					EnableMenuItem(actionMenu, ID_STOP_BUTTON, MF_BYCOMMAND | MF_GRAYED);
					 EnableWindow(startButton, true);
					EnableMenuItem(actionMenu, ID_START_BUTTON, MF_BYCOMMAND | MF_ENABLED);
					 setEnabledControls(true);
					 paused = false;
					 SetPauseButtonLabel();

					 SetFocus(startButton);
					 repaint();
					 break;
					}
				 break;

			 case ID_PAUSE_BUTTON:
				 switch (HIWORD(wParam))
				 {
				 case BN_CLICKED:
					 if (!canMove)
						 break;
					 
					 if (paused)
					 {
						 paused = false;
						 Start();
					 }
					 else
					 {
						 paused = true;
						 Stop();
					 }
					 SetPauseButtonLabel();
					 break;
				 }
				 break;

			 case ID_WIDTH_EDIT:
				 switch (HIWORD(wParam))
				 {
				 case EN_KILLFOCUS:
						char text[4];
						GetWindowText(widthEdit, text, 4);
						int newWidth = atoi(text);

            if (newWidth != grid->GetWidth())
            {
                grid->SetWidth(newWidth);
                gridResized = true;
								repaint();
            }
						sprintf(text, "%d", grid->GetWidth());
						SetWindowText(widthEdit, text);

					 break;
				 }
				 break;

			 case ID_HEIGHT_EDIT:
				 switch (HIWORD(wParam))
				 {
				 case EN_KILLFOCUS:
						char text[4];
						GetWindowText(heightEdit, text, 4);
						int newHeight = atoi(text);

            if (newHeight != grid->GetHeight())
            {
                grid->SetHeight(newHeight);
                gridResized = true;
								repaint();
            }
						sprintf(text, "%d", grid->GetHeight());
						SetWindowText(heightEdit, text);

					 break;
				 }
				 break;

			 case ID_SPEED_EDIT:
				 switch (HIWORD(wParam))
				 {
				 case EN_KILLFOCUS:

						char text[4];
						GetWindowText(speedEdit, text, 4);
						int newSpeedText = atoi(text);

            if ((newSpeedText < 1) || (newSpeedText > 10))
						{
							MessageBox(hwnd, TEXT("Speed must be number 1 - 10."), TEXT("Error"), 0);
						}
            else
						{
                switch (caterpillarSpeed = newSpeedText)
                {
                    case 1:  caterpillarSpeedTime = 500; break;
                    case 2:  caterpillarSpeedTime = 350; break;
                    case 3:  caterpillarSpeedTime = 250; break;
                    case 4:  caterpillarSpeedTime = 180; break;
                    case 5:  caterpillarSpeedTime = 130; break;
                    case 6:  caterpillarSpeedTime = 100; break;
                    case 7:  caterpillarSpeedTime = 80; break;
                    case 8:  caterpillarSpeedTime = 67; break;
                    case 9:  caterpillarSpeedTime = 53; break;
                    case 10: caterpillarSpeedTime = 45; break;
								}
						}
						sprintf(text, "%d", caterpillarSpeed);
						SetWindowText(speedEdit, text);
					 break;
				 }
				 break;

			 case ID_BARRIERS_EDIT:
				 switch (HIWORD(wParam))
				 {
				 case EN_KILLFOCUS:
						char text[4];
						GetWindowText(barriersEdit, text, 4);
						int newNumberOfBarriers = atoi(text);

            int currentMaxNumberOfBarriers = (grid->GetHeight() * grid->GetWidth() / 10);
            if ((newNumberOfBarriers < 0) || 
                (newNumberOfBarriers > currentMaxNumberOfBarriers))
            {
							char err[50];
							sprintf(err, "# of barriers must be currently number 0 - %d", currentMaxNumberOfBarriers);
							MessageBox(hwnd, TEXT(err), TEXT("Error"), 0);
            }
            else
            {
                numberOfBarriers = newNumberOfBarriers;
            }
						sprintf(text, "%d", numberOfBarriers);
						SetWindowText(barriersEdit, text);
					 break;
				 }
				 break;
				 
			 }
			 return 0;

		 case WM_KEYDOWN:
			 if (caterpillar != NULL)
			 {
				 switch (wParam)
				 {
				 case VK_LEFT:
					 caterpillar->SetMoveDirection(caterpillar->TO_LEFT);
					 break;

				 case VK_RIGHT:
					 caterpillar->SetMoveDirection(caterpillar->TO_RIGHT);
					 break;

				 case VK_UP:
					 caterpillar->SetMoveDirection(caterpillar->TO_UP);
					 break;

				 case VK_DOWN:
					 caterpillar->SetMoveDirection(caterpillar->TO_DOWN);
					 break;
				 }
			 }
			 canMove = true;
			 return  0;

		 case WM_TIMER:
            if (canMove) // test needed, when killing applet with running
                                        // caterpillar g can be null
            {
                caterpillar->Move(mainWindow);
            }
            
            if (!caterpillar->IsCorrectHeadPosition())
            {
                SendMessage(mainWindow, WM_COMMAND, MAKELONG(ID_STOP_BUTTON, BN_CLICKED), (LPARAM) stopButton);
                MessageBox(mainWindow, TEXT("CRASH!!! Sorry, this is the end of the game :o)"), TEXT("Game Over"), 0);
								SetFocus(startButton);
						}
			 return 0;

     case WM_DESTROY:
					delete grid;
					if (caterpillar)
						delete caterpillar;
          PostQuitMessage(0);
          return 0;
     }
     return DefWindowProc (hwnd, message, wParam, lParam) ;
}

LRESULT CALLBACK MyButtonWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_KEYDOWN:
		switch (wParam)
		{
		case VK_LEFT:
      caterpillar->SetMoveDirection(caterpillar->TO_LEFT);
			break;
		case VK_RIGHT:
      caterpillar->SetMoveDirection(caterpillar->TO_RIGHT);
			break;
		case VK_UP:
      caterpillar->SetMoveDirection(caterpillar->TO_UP);
			break;
		case VK_DOWN:
      caterpillar->SetMoveDirection(caterpillar->TO_DOWN);
			break;
		case VK_RETURN:
		case VK_SPACE:
			unsigned short us;
			if (hwnd == startButton)
				us = ID_START_BUTTON;
			else if (hwnd == stopButton)
				us = ID_STOP_BUTTON;
			else
				us = ID_PAUSE_BUTTON;

			SendMessage(mainWindow, WM_COMMAND, MAKELONG(us, BN_CLICKED), (LPARAM) hwnd);
			return 0;
			
		case VK_TAB:
			MoveFocus(hwnd);
			return 0;
		}
		if (canMove == false)
		{
			canMove = true;
			repaint();
		}
		return 0;
	}

	return CallWindowProc(oldButtonProc, hwnd, message, wParam, lParam);
}

LRESULT CALLBACK MyEditWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_KEYDOWN:
		switch (wParam)
		{
		case VK_TAB:
			MoveFocus(hwnd);
			break;
		}
	}

	return CallWindowProc(oldEditProc, hwnd, message, wParam, lParam);
}
